-- Talent check file for the warrior class
local class = "WARRIOR";

-- set up check table
ScrubFork.checks = ScrubFork.checks or { };
ScrubFork.checks[class] = ScrubFork.checks[class] or { };

-- simple function to create new checks
local function check(name, severity, check)
	tinsert(ScrubFork.checks[class], {name=name, severity=severity, check=check});
end

local GREEN = ScrubFork.GREEN;
local YELLOW = ScrubFork.YELLOW;
local RED = ScrubFork.RED;

-- Meta
check("Partial Protection Spec",		YELLOW,	function (talents) return (talents:spent(3) >= 21) and (talents:spent(3) < 41); end);
check("2H Spec with DW Spec",			RED,	function (talents) return (talents:rank(1, 10) > 0) and (talents:rank(2,  9) > 0); end);
check("Mortal Strike without Imp. Slam",YELLOW,	function (talents) return (talents:rank(1, 20) > 0) and (talents:rank(2, 12) < 2); end);
check("Bloodthirst with 2H Spec",		YELLOW,	function (talents) return (talents:rank(2, 18) > 0) and (talents:rank(1, 10) > 0); end);
check("Multiple Weapon Specs",			RED,	function (talents)
													local axe   = ((talents:rank(1, 12) > 0) and 1) or 0;
													local mace  = ((talents:rank(1, 14) > 0) and 1) or 0;
													local sword = ((talents:rank(1, 15) > 0) and 1) or 0;
													return axe + mace + sword > 1;
												end);


-- Arms
check("Lacking Deflection",				YELLOW,	function (talents) return (talents:spent(3) >= 15) and (talents:rank(1,  2) < 5); end);
check("Improved Rend",					RED,	function (talents) return talents:rank(1,  3) > 0; end);
check("Deep Wounds without Impale",		RED,	function (talents) return (talents:rank(1,  9) > 0) and (talents:rank(1, 11) < 2); end);
check("Lacking 2H Specialization",		RED,	function (talents) return (talents:spent(1) >= 23) and (talents:rank(1, 10) < 5); end);
check("Lacking Impale",					RED,	function (talents) return (talents:spent(1) >= 17) and (talents:rank(1, 11) < 2); end);
check("No Deathwish",					RED,	function (talents) return (talents:spent(1) >= 21) and (talents:rank(1, 13) < 1); end);
check("Partial Poleaxe Specialization",	YELLOW,	function (talents) return (talents:spent(1) >= 28) and (talents:rank(1, 12) > 0) and (talents:rank(1, 12) < 5); end);
check("Partial Mace Specialization",	YELLOW,	function (talents) return (talents:spent(1) >= 28) and (talents:rank(1, 14) > 0) and (talents:rank(1, 14) < 5); end);
check("Partial Sword Specialization",	YELLOW,	function (talents) return (talents:spent(1) >= 28) and (talents:rank(1, 15) > 0) and (talents:rank(1, 15) < 5); end);
check("Improved Hamstring (PvP)",		YELLOW,	function (talents) return talents:rank(1, 17) > 0; end);
check("Lacking Blood Frenzy",			RED,	function (talents) return (talents:spent(1) >= 33) and (talents:rank(1, 19) < 2); end);
check("No Mortal Strike",				RED,	function (talents) return (talents:spent(1) >= 31) and (talents:rank(1, 20) < 1); end);
check("Second Wind (PvP)",				YELLOW,	function (talents) return talents:rank(1, 21) > 0; end);
check("Improved Mortal Strike",			YELLOW,	function (talents) return talents:rank(1, 22) > 0; end);
check("Endless Rage",					YELLOW,	function (talents) return talents:rank(1, 23) > 0; end);

-- Fury
check("Booming Voice (PvP)",			YELLOW,	function (talents) return talents:rank(2,  1) > 0; end);
check("Lacking Cruelty",				RED,	function (talents) return (talents:spent(3) <= 15) and (talents:rank(2,  2) < 5); end);
check("Improved Cleave",				YELLOW,	function (talents) return talents:rank(2,  5) > 0; end);
check("Blood Craze (PvP)",				YELLOW,	function (talents) return talents:rank(2,  7) > 0; end);
check("Lacking Commanding Presence",	YELLOW,	function (talents) return (talents:spent(2) >= 16) and (talents:rank(2,  8) < 4); end);
check("Enrage without Flurry",			RED,	function (talents) return (talents:rank(2, 11) > 0) and (talents:rank(2, 16) < 3); end);
check("No Sweeping Strikes",			RED,	function (talents) return (talents:spent(2) >= 21) and (talents:rank(2, 13) < 1); end);
check("Lacking Weapon Mastery",			YELLOW,	function (talents) return (talents:spent(2) >= 23) and (talents:rank(2, 14) < 2); end);
check("Lacking Flurry",					RED,	function (talents) return (talents:spent(2) >= 30) and (talents:rank(2, 16) < 5); end);
check("No Bloodthirst",					RED,	function (talents) return (talents:spent(2) >= 31) and (talents:rank(2, 18) < 1); end);
check("Lacking Precision",				YELLOW,	function (talents) return (talents:spent(2) >= 34) and (talents:rank(2, 17) < 3); end);
check("Lacking Imp. Berserker Stance",	RED,	function (talents) return (talents:spent(2) >= 40) and (talents:rank(2, 20) < 5); end);

-- Protection
check("Lacking Shield Specialization",	RED,	function (talents) return (talents:spent(3) >= 11) and (talents:rank(3,  4) < 5); end);
check("Lacking Toughness",				RED,	function (talents) return (talents:spent(3) >= 20) and (talents:rank(3,  5) < 5); end);
check("No Last Stand",					RED,	function (talents) return (talents:spent(3) >= 11) and (talents:rank(3,  6) < 1); end);
check("No Improved Shield Block",		RED,	function (talents) return (talents:spent(3) >= 15) and (talents:rank(3,  7) < 1); end);
check("Improved Revenge",				YELLOW,	function (talents) return talents:rank(3,  8) > 0; end);
check("Lacking Defiance",				RED,	function (talents) return (talents:spent(3) >= 15) and (talents:rank(3,  9) < 3); end);
check("Improved Disarm (PvP)",			YELLOW,	function (talents) return talents:rank(3, 11) > 0; end);
check("No Concussion Blow",				RED,	function (talents) return (talents:spent(3) >= 21) and (talents:rank(3, 14) < 1); end);
check("Improved Shield Bash (PvP)",		YELLOW,	function (talents) return talents:rank(3, 15) > 0; end);
--check("Lacking 1H Weapon Spec",			YELLOW,	function (talents) return (talents:spent(3) >= 43) and (talents:rank(3, 17) < 5); end);
check("Lacking Imp. Defensive Stance",	YELLOW,	function (talents) return (talents:spent(3) >= 38) and (talents:rank(3, 18) < 3); end);
check("No Shield Slam",					RED,	function (talents) return (talents:spent(3) >= 31) and (talents:rank(3, 19) < 1); end);
check("Lacking Focused Rage",			YELLOW,	function (talents) return (talents:spent(3) >= 38) and (talents:rank(3, 20) < 3); end);
check("Lacking Vitality",				YELLOW,	function (talents) return (talents:spent(3) >= 44) and (talents:rank(3, 21) < 5); end);
check("No Devastate",					RED,	function (talents) return (talents:spent(3) >= 41) and (talents:rank(3, 22) < 1); end);
