-- ScrubFork is based off of the original Scrub mod, forked as of  Aug 07, 2008
-- ScrubFork is an addon that scans raid members and flags poor PVE talent
-- choices for further inspection.

-- Command options
local options = {
    type = "group",
    args = {
        raid = {
            name = "Raid",
            desc = "Scan and display raid talents",
            type = "execute",
            func = "InitiateRaidScan",
        },
        party = {
            name = "Party",
            desc = "Scan and display party talents",
            type = "execute",
            func = "InitiatePartyScan",
        },
        target = {
            name = "Target",
            desc = "Scan and display target talents",
            type = "execute",
            func = "InitiateTargetScan",
        },
        show = {
            name = "Show",
            desc = "Show the report with cached data",
            type = "execute",
            func = "Display",
        },
    }
};

-- Set up addon
ScrubFork = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0");
ScrubFork:RegisterChatCommand("/ScrubFork", options);

ScrubFork.UNKNOWN = 0;
ScrubFork.GREEN = 1;
ScrubFork.YELLOW = 2;
ScrubFork.RED = 3;

ScrubFork.colors = { };
ScrubFork.colors[ScrubFork.UNKNOWN]    = "888888";
ScrubFork.colors[ScrubFork.GREEN]    = "00FF00";
ScrubFork.colors[ScrubFork.YELLOW]    = "FFFF00";
ScrubFork.colors[ScrubFork.RED]        = "FF0000";
ScrubFork.colors["DRUID"]        = "FF7D0A";
ScrubFork.colors["HUNTER"]        = "ABD473";
ScrubFork.colors["MAGE"]        = "69CCF0";
ScrubFork.colors["PRIEST"]        = "FFFFFF";
ScrubFork.colors["PALADIN"]        = "F58CBA";
ScrubFork.colors["ROGUE"]        = "FFF569";
ScrubFork.colors["SHAMAN"]        = "2459FF";
ScrubFork.colors["WARLOCK"]        = "9482CA";
ScrubFork.colors["WARRIOR"]        = "C79C6E";

ScrubFork.icons = { };
ScrubFork.icons["?"] = "Interface\\Icons\\INV_Misc_QuestionMark";
ScrubFork.icons["Hybrid"] = "Interface\\Icons\\Spell_Nature_ElementalAbsorption";
ScrubFork.icons["Classes"] = "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes";

ScrubFork.icons["DRUID"]   = {0.7421875, 0.98828125, 0, 0.25};
ScrubFork.icons["HUNTER"]  = {0, 0.25, 0.25, 0.5};
ScrubFork.icons["MAGE"]    = {0.25, 0.49609375, 0, 0.25};
ScrubFork.icons["PRIEST"]  = {0.49609375, 0.7421875, 0.25, 0.5};
ScrubFork.icons["PALADIN"] = {0, 0.25, 0.5, 0.75};
ScrubFork.icons["ROGUE"]   = {0.49609375, 0.7421875, 0, 0.25};
ScrubFork.icons["SHAMAN"]  = {0.25, 0.49609375, 0.25, 0.5};
ScrubFork.icons["WARLOCK"] = {0.7421875, 0.98828125, 0.25, 0.5};
ScrubFork.icons["WARRIOR"] = {0, 0.25, 0, 0.25};
                

ScrubFork.talents = { };

-- OnEnable
function ScrubFork:OnEnable()
    
    self:SetupFrame();
    
end

-- Debugging function, check veracity of all functions
function ScrubFork:TestChecks()
    
    self:Print("Testing talent checks for all classes...");
    
    if self.TalentChecks then self:Print("Outdated table ScrubFork.TalentChecks found!"); end
    
    if not self.checks then
        self:Print("No checks found!");
        return;
    end
    
    local talents = { 
        name = "Test Unit",
        class = "DEATHKNIGHT",
        level = 70,
        tree = { },
        spent = function () return 61; end,
        rank = function () return 0; end,
    };
    
    for class, checks in pairs(self.checks) do
        self:Print("Testing "..strlower(class).." checks...");
        for _, check in ipairs(checks) do check.check(talents); end
    end
    
    self:Print("Testing completed successfuly.");
    
end

-- Functions set during setup of talent tables
local function spent(self, tree) return (self.tree[tree] and self.tree[tree].spent) or -1; end
local function rank(self, tree, index) return (self.tree[tree] and self.tree[tree].talent[index]) or -1; end


----------------------
--     SCANNING     --
----------------------

-- Check whether a target is in range or not
local function inRange(unit) return UnitIsVisible(unit) or false; end
-- local function inRange(unit) return CheckInteractDistance(unit, 1) or false; end

-- Store talent information once it becomes available
function ScrubFork:RecordTalents()
    
    local unit = self.scanning_unit;
    if self.scanning then
        unit = unit..self.scanning;
    end
    
    local unitIsNotPlayer = not UnitIsUnit("player", unit);
    -- prevent further interruption
    if unitIsNotPlayer then -- TODO: This check can probably be done better
        self:UnregisterEvent("INSPECT_TALENT_READY");
    end
    
    -- set up default table values
    local talents = {
        name = UnitName(unit),
        class = select(2, UnitClass(unit)),
        level = UnitLevel(unit),
        tree = { },
        spent = spent,
        rank = rank,
    };
    
    -- range check
    if not inRange(unit) then
        talents.unknown = true;
        return talents;
    end
    
    -- for each talent tab
    for tab = 1, GetNumTalentTabs(unitIsNotPlayer) do
        
        -- record tab
        local name, icon, spent = GetTalentTabInfo(tab, unitIsNotPlayer);
        talents.tree[tab] = {name=name, icon=icon, spent=spent, talent={ }};
        
        -- record talent choices
        for talent = 1, GetNumTalents(tab, unitIsNotPlayer) do
            talents.tree[tab].talent[talent] = select(5, GetTalentInfo(tab, talent, unitIsNotPlayer));
        end
    
    end
    
    return talents;
    
end

-- Initiate a raidwide talent scan
function ScrubFork:InitiateRaidScan()
    
    -- abort if player is dead
    if UnitIsDead("player") then
        self:Print("Cannot perform scans while dead");
        return;
    end
    
    -- abort if scan in progress
    if self.scanning_unit then
        self:Print("Cannot perform multiple scans simultaneously");
        return;
    end
    
    -- abort unless we are in a raid
    if GetNumRaidMembers() == 0 then
        self:Print("You are not in a raid group!");
        return;
    end
    
    -- clear old data
    self.talents = { };
    
    self:Print("Beginning scan...");
    
    -- set current scanning index and begin scan
    self.scanning = 1;
    self.scanning_unit = "raid";
    self:ScanNext();
    
end

-- Initiate a partywide talent scan
function ScrubFork:InitiatePartyScan()
    
    -- abort if player is dead
    if UnitIsDead("player") then
        self:Print("Cannot perform scans while dead");
        return;
    end
    
    -- abort if scan in progress
    if self.scanning_unit then
        self:Print("Cannot perform multiple scans simultaneously");
        return;
    end
    
    -- abort unless we are in a party
    if GetNumPartyMembers() == 0 then
        self:Print("You are not in a group!");
        return;
    end
    
    -- clear old data
    self.talents = { };
    
    self:Print("Beginning scan...");
    
    -- set current scanning index and begin scan
    self.scanning = 1;
    self.scanning_unit = "party";
    self:ScanNext();
    
end

-- Scan the next raid unit
function ScrubFork:ScanNext()
    
    if not self.scanning then return; end
    
    local id = self.scanning;
    local unit = self.scanning_unit..id;
    
    -- check that unit exists
    if not UnitExists(unit) then
        self.talents[id] = nil;
        self:Iterate();
        return;
    end
    
    -- check range
    if inRange(unit) then

        if UnitIsUnit("player", unit) then
            ScrubFork.talents[ScrubFork.scanning] = ScrubFork:RecordTalents();
            ScrubFork:Iterate();
        else
            self:RegisterEvent("INSPECT_TALENT_READY",
                function ()
                    ScrubFork.talents[ScrubFork.scanning] = ScrubFork:RecordTalents();
                    ScrubFork:Iterate();
                end);
            NotifyInspect(unit);
        end
        
    else
        
        -- set up default table values
        self.talents[id] = {
            name = UnitName(unit),
            class = select(2, UnitClass(unit)),
            level = UnitLevel(unit),
            tree = { },
            spent = spent,
            rank = rank,
            unknown = true;
        };
        
        self:Iterate();
        
    end
    
end

-- End-of-function iteration check
function ScrubFork:Iterate()
    
    if self.scanning and
        (((self.scanning_unit == "raid") and (self.scanning < 40)) or
         ((self.scanning_unit == "party") and (self.scanning < 5))) then
        
        -- iterate to the next unit id
        self.scanning = self.scanning + 1;
        self:ScanNext();
        
    else
        
        -- wrap it up, we've scanned through all units
        self.scanning = nil;
        self.scanning_unit = nil;
        self:Print("Scan complete");
        self:AnalyzeAllTalents();
        self:UpdateReport();
        self:Display();
        
    end
    
end

-- Scan the current target
function ScrubFork:InitiateTargetScan()
    
    -- abort if player is dead
    if UnitIsDead("player") then
        self:Print("Cannot perform scans while dead");
        return;
    end
    
    -- abort if scan in progress
    if self.scanning_unit then
        self:Print("Cannot perform multiple scans simultaneously");
        return;
    end
    
    if not UnitExists("target") then
        self:Print("You have no target");
        return;
    end
    
    if not inRange("target") then
        self:Print("You are too far away");
        return;
    end
    
    self.scanning = nil;
    self.scanning_unit = "target";
    
    if UnitIsUnit("player", "target") then
        self:RecordTargetTalents();
    else
        self:RegisterEvent("INSPECT_TALENT_READY", "RecordTargetTalents");
        NotifyInspect("target");
    end
    
end

-- Wrapper for scanning current target
function ScrubFork:RecordTargetTalents()
    
    local talents = self:RecordTalents();
    self.scanning = nil;
    self.scanning_unit = nil;
    
    self:AnalyzeTalents(talents);
    
    local name = "|cFF"..self.colors[talents.class]..talents.name.."|r";
    
    if talents.unknown then
        self:Print(name.." - Unknown Spec ?/?/?");
    else
        self:Print(name.." - "..talents.specname.."  "..talents:spent(1).."/"..talents:spent(2).."/"..talents:spent(3));
        
        if getn(talents.report) == 0 then
            self:Print("|cFF"..self.colors[self.GREEN].."No issues found|r");
        end
        
        for _, result in ipairs(talents.report) do
            
            local value = result[1];
            local description = result[2];
            
            self:Print("|cFF"..self.colors[value]..description.."|r");
            
        end
        
    end
    
end

----------------------
--     ANALYSIS     --
----------------------

-- Analyze an individual talent set
function ScrubFork:AnalyzeTalents(talents)
    
    if not talents or talents.unknown then return; end
    
    -- determine spec and spec icon
    local total = 0;
    for _, tree in ipairs(talents.tree) do total = total + tree.spent; end
    local threshold = total*0.4;
    
    local deeptrees = 0;
    for _, tree in ipairs(talents.tree) do if tree.spent >= threshold then deeptrees = deeptrees + 1; end end
    
    if deeptrees ~= 1 then
        talents.specname = "Hybrid";
        talents.specicon = self.icons["Hybrid"];
    else
        local deepcount = 0;
        local deeptree = nil;
        for _, tree in ipairs(talents.tree) do
            if tree.spent > deepcount then
                deepcount = tree.spent;
                deeptree = tree;
            end
        end
        talents.specname = deeptree.name;
        talents.specicon = deeptree.icon;
    end
    
    -- initialize report section
    talents.report = { };
    
    -- make sure they have spent all their talent points...
    if talents.level > (total + 9) then
        tinsert(talents.report, {self.RED, "Unspent talent points"});
    end
    
    -- load class checks
    local checks = self.checks[talents.class] or { };
    
    -- execute checks
    for _, check in ipairs(checks) do
        
        self:ExecuteTalentCheck(check, talents);
        
    end
    
end

-- Run an individual test on a talent spec
function ScrubFork:ExecuteTalentCheck(check, talents)
    
    -- run check in protected mode
    local success, result = pcall(check.check, talents);
    
    -- catch errors
    if not success then
        self:Print("|cFFFF0000Error|r executing talent check |cFFFFFF00["..check.name.."]|r Please report to Slake: "..result);
        return;
    end
    
    local value = (result and check.severity) or self.GREEN;
    
    -- if the test failed, add the warning
    if value ~= self.GREEN then
        tinsert(talents.report, {value, check.name});
    end
    
end

-- Analyze recorded talents
function ScrubFork:AnalyzeAllTalents()
    
    -- abort if still scanning
    if self.scanning then return; end
    
    -- for each talent set
    for id, talents in pairs(self.talents) do
        
        self:AnalyzeTalents(talents);
        
    end
    
end




-----------------------
--     INTERFACE     --
-----------------------

-- Initialize the report frame
function ScrubFork:SetupFrame()
    
    -- Create the frame, store local reference
    local f = GameFontNormal:GetFont();
    local report = CreateFrame("Frame", "ScrubForkReportFrame", UIParent);
    self.report = report;
    
    -- Set the frame up
    report:Hide();
    report:SetWidth(320);
    report:SetHeight(420);
    report:EnableMouse(true);
    report:SetMovable(true);
    report:RegisterForDrag("LeftButton");
    report:SetScript("OnDragStart", function() this:StartMoving() end);
    report:SetScript("OnDragStop", function() this:StopMovingOrSizing() end);
    report:SetBackdrop({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true, tileSize = 16,
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
        insets = {left = 4, right = 4, top = 4, bottom = 4},
    });
    report:SetBackdropBorderColor(.5, .5, .5);
    report:SetBackdropColor(0, 0, 0);
    report:ClearAllPoints();
    report:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
    tinsert(UISpecialFrames, "ScrubForkReportFrame");
    
    -- Gradient background
    local rfade = report:CreateTexture(nil, "BORDER");
    rfade:SetWidth(314);
    rfade:SetHeight(25);
    rfade:SetTexture("Interface\\ChatFrame\\ChatFrameBackground");
    rfade:SetPoint("TOP", report, "TOP", 0, -4);
    rfade:SetBlendMode("ADD");
    rfade:SetGradientAlpha("VERTICAL", .1, .1, .1, 0, .25, .25, .25, 1);
    
    -- Title
    local header = report:CreateFontString(nil,"OVERLAY");
    header:SetFont(f, 14);
    header:SetWidth(320);
    header:SetText("ScrubFork");
    header:SetTextColor(1, .8, 0);
    header:ClearAllPoints();
    header:SetPoint("TOP", report, "TOP", 0, -10);
    report.header = header;
    
    -- Create individual report frames
    report.units = { };
    
    local function CreateUnitReportFrame(id)
        
        local unit = CreateFrame("Frame", report:GetName().."Unit"..id, report);
        --unit:Hide();
        unit:SetWidth(120);
        unit:SetHeight(16);
        unit:EnableMouse(true);
        unit:SetScript("OnEnter", function () ScrubFork:ReportTooltip(id); end);
        unit:SetScript("OnLeave", function () GameTooltip:Hide(); end);
        unit:ClearAllPoints();
        
        local icon = unit:CreateTexture();
        icon:SetWidth(16);
        icon:SetHeight(16);
        icon:SetPoint("LEFT", unit, "LEFT");
        icon:SetTexture(self.icons["Classes"]);
        unit.classicon = icon;
        
        icon = unit:CreateTexture();
        icon:SetWidth(16);
        icon:SetHeight(16);
        icon:SetPoint("LEFT", unit.classicon, "RIGHT");
        icon:SetTexture(self.icons["?"]);
        unit.specicon = icon;
        
        local text = unit:CreateFontString(nil, "OVERLAY");
        text:SetFont(f, 12);
        text:SetWidth(90);
        text:SetHeight(14);
        text:SetJustifyV("CENTER");
        text:SetJustifyH("LEFT");
        text:SetPoint("LEFT", unit.specicon, "RIGHT", 5, 0);
        text:SetText("raid"..id);
        unit.text = text;
        
        report.units[id] = unit;
        
        return unit;
        
    end
    
    CreateUnitReportFrame(1):SetPoint("TOPLEFT", report, "TOPLEFT", 30, -30);
    for id = 2, 20 do
        CreateUnitReportFrame(id):SetPoint("TOPLEFT", report.units[id-1], "BOTTOMLEFT", 0, -1);
    end
    CreateUnitReportFrame(21):SetPoint("TOPLEFT", report.units[1], "TOPRIGHT", 10, 0);
    for id = 22, 40 do
        CreateUnitReportFrame(id):SetPoint("TOPLEFT", report.units[id-1], "BOTTOMLEFT", 0, -1);
    end
    
    -- Close button
    local closebutton = CreateFrame("Button", nil, report);
    closebutton.owner = self;
    closebutton:SetWidth(125);
    closebutton:SetHeight(32);
    closebutton:SetPoint("BOTTOM", report, "BOTTOM", 0, 10);
    report.closebutton = closebutton;
    
    local t = closebutton:CreateTexture();
    t:SetWidth(125);
    t:SetHeight(32);
    t:SetPoint("CENTER", closebutton, "CENTER");
    t:SetTexture("Interface\\Buttons\\UI-Panel-Button-Up");
    t:SetTexCoord(0, 0.625, 0, 0.6875);
    closebutton:SetNormalTexture(t);
    
    t = closebutton:CreateTexture(nil, "BACKGROUND");
    t:SetTexture("Interface\\Buttons\\UI-Panel-Button-Down");
    t:SetTexCoord(0, 0.625, 0, 0.6875);
    t:SetAllPoints(closebutton);
    closebutton:SetPushedTexture(t);
    
    t = closebutton:CreateTexture();
    t:SetTexture("Interface\\Buttons\\UI-Panel-Button-Highlight");
    t:SetTexCoord(0, 0.625, 0, 0.6875);
    t:SetAllPoints(closebutton);
    t:SetBlendMode("ADD");
    closebutton:SetHighlightTexture(t);
    
    local closebuttontext = closebutton:CreateFontString(nil,"OVERLAY");
    closebuttontext:SetFontObject(GameFontHighlight);
    closebuttontext:SetText("Close");
    closebuttontext:SetAllPoints(closebutton);
    
    report.closebutton:SetScript("OnClick", function() report:Hide(); end);
    
end

-- Display report frame
function ScrubFork:Display()
    if not self.scanning then
        self.report:Show();
    end
end

-- Update the report frame
function ScrubFork:UpdateReport()
    
    -- abort if still scanning
    if self.scanning then return; end
    
    -- iterate through raid units
    for id = 1, 40 do
        
        local unit = self.report.units[id];
        local talents = self.talents[id];
        
        if talents then
            
            unit.classicon:SetTexCoord(unpack(self.icons[talents.class]));
            
            if talents.unknown then
                
                unit.specicon:SetTexture(self.icons["?"]);
                unit.text:SetText("|cFF"..self.colors[self.UNKNOWN]..talents.name.."|r");
                
            else
            
                -- figure out severity of report
                local value = self.UNKNOWN;
                
                if talents.report then
                    value = self.GREEN;
                    for _, result in ipairs(talents.report) do
                        value = max(value, result[1]);
                    end
                end
                
                -- set up unit
                unit.specicon:SetTexture(talents.specicon);
                unit.text:SetText("|cFF"..self.colors[value]..talents.name.."|r");
                
            end
            
            unit:Show();
            
        else
            
            unit:Hide();
            
        end
        
    end

end

-- Generate and show a tooltip for a report
function ScrubFork:ReportTooltip(id)
    
    -- get local references
    local unit = self.report.units[id];
    local talents = self.talents[id];
    local tooltip = GameTooltip;
    
    -- set up tooltip
    tooltip:SetOwner(unit, "ANCHOR_CURSOR");
    
    if not talents then
        tooltip:AddLine("|cFF"..self.colors[self.UNKNOWN].."Unknown Name|r");
        tooltip:AddLine("Unknown Spec ?/?/?");
        tooltip:AddTexture(self.icons["?"]);
        tooltip:Show();
        return;
    end
    
    -- fill in tooltip
    tooltip:AddLine("|cFF"..self.colors[talents.class]..talents.name.."|r");
    -- tooltip:AddTexture(self.icons[talents.class]); doesn't seem to work
    
    if talents.unknown then
        tooltip:AddLine("Unknown Spec  ?/?/?");
        tooltip:AddTexture(self.icons["?"]);
    else
        tooltip:AddLine(talents.specname.."  "..talents:spent(1).."/"..talents:spent(2).."/"..talents:spent(3));
        tooltip:AddTexture(talents.specicon);
        
        tooltip:AddLine("\nReport:");
        
        if getn(talents.report) == 0 then
            tooltip:AddLine("|cFF"..self.colors[self.GREEN].."No issues found|r");
        end
        
        for _, result in ipairs(talents.report) do
            
            local value = result[1];
            local description = result[2];
            
            tooltip:AddLine("|cFF"..self.colors[value]..description.."|r");
            
        end
        
    end
    
    tooltip:Show();
    
end
